==> pam_tcfs-1.0.tar.gz: pam_tcfs-1.0/src/tcfs_chgkey.c <==
/*
 *      pam_tcfs.so v1.0
 *
 *      written by Luigi Catuogno [luicat@mikonos.dia.unisa.it]
 *
 *      Jun 30 1997
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <pwd.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <syslog.h>
#include "tcfscode.h"
#include "defines.h"
#include "linux/kdes.h"
#include "tcfspasswd.h"

int ex_lockfile()
{
	if(access(TCFSPWDTMP, 0) == 0)
	{
		syslog(LOG_ERR,"/etc/tcfstmp exists, can't update key file");
		return 0;
	}
	else return 1;
}

int tcfs_chgkey(char *user, char *oldp, char *newp)
{
	static char old[13],new[13];
	char *key;
	FILE *tcfstmp, *tcfspwd;
	int r,dummy;
	des_key_schedule ks;
        static struct tcfspasswd e;
	char entry[MaxLineLen];

	
	bzero(old,10);
	bzero(new,10);
        strcpy(old,oldp);
	strcpy(new,newp);

	key=(char*)malloc(KEYSIZE);
	if(!key)
	  {
		syslog(LOG_INFO,"malloc failed");
		exit(1);
	  }
	bzero(key,KEYSIZE);

	if(!(tcfstmp = fopen(TCFSPWDTMP, "w")))
	{
		syslog(LOG_ERR,"Can't open /etc/tcfstmp, can't update key file");
		return 0;
	}

	tcfspwd=fopen (TCFSPWDFILE, "r");
	
	while (!feof(tcfspwd))
	{
		
		fgets (entry, MaxLineLen, tcfspwd);
		if (feof (tcfspwd))
			break;

		if (strncmp(entry, user, strlen(user))!=0)
		{
			fwrite (entry, 1, strlen(entry), tcfstmp);
		}
		else
		{
			if(!tcfs_strgetpwnam(entry,&e))
				return 0;

			tcfs_key(user,old,e.upw,&key);

			if (strlen (new)<8)
				strncat (new, new, strlen(new));

			
			des_set_key ((des_cblock *)new, ks);
			des_ecb_encrypt ((des_cblock *)key, (des_cblock *)key, ks, DES_ENCRYPT);
			fprintf (tcfstmp, "%s:%s\n", user, 
				tcfs_encode(key, KEYSIZE));
		}			
	}
	
	unlink(TCFSPWDOLD);
	link(TCFSPWDFILE, TCFSPWDOLD);
	unlink(TCFSPWDFILE);
	link(TCFSPWDTMP, TCFSPWDFILE);
	unlink(TCFSPWDTMP); 
	chmod(TCFSPWDFILE, 0644);
	syslog(LOG_INFO,"tcfspasswd updated");	
	free(key);
	return 1;
}

==> pam_tcfs-1.0.1.tar.gz: pam_tcfs-1.0.1/src/tcfs_chgkey.c <==
/*
 *      pam_tcfs.so v1.0.1
 *
 *      written by Luigi Catuogno [luicat@mikonos.dia.unisa.it]
 *
 *      Jul 2 1997
 */

#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include <termios.h>
#include <pwd.h>
#include <ctype.h>
#include <time.h>
#include <string.h>
#include <syslog.h>
#include "tcfscode.h"
#include "defines.h"
#include "linux/kdes.h"
#include "tcfspasswd.h"

int ex_lockfile()
{
	if(access(TCFSPWDTMP, 0) == 0)
	{
		syslog(LOG_ERR,"/etc/tcfstmp exists, can't update key file");
		return 0;
	}
	else return 1;
}

int tcfs_chgkey(char *user, char *oldp, char *newp)
{
	static char old[UUKEYSIZE],new[UUKEYSIZE];
	char *key;
	FILE *tcfstmp, *tcfspwd;
	int r,dummy,i=0;
	des_key_schedule ks;
        static struct tcfspasswd e;
	char entry[MaxLineLen];

	
	bzero(old,10);
	bzero(new,10);
        strcpy(old,oldp);
	strcpy(new,newp);

	key=(char*)malloc(UUKEYSIZE);
	if(!key)
	  {
		syslog(LOG_INFO,"malloc failed");
		exit(1);
	  }
	bzero(key,UUKEYSIZE);

	if(!(tcfstmp = fopen(TCFSPWDTMP, "w")))
	{
		syslog(LOG_ERR,"Can't open /etc/tcfstmp, can't update key file");
		return 0;
	}

	tcfspwd=fopen (TCFSPWDFILE, "r");
	
	while (!feof(tcfspwd))
	{
		
		fgets (entry, MaxLineLen, tcfspwd);
		if (feof (tcfspwd))
			break;

		if (strncmp(entry, user, strlen(user))!=0)
		{
			fwrite (entry, 1, strlen(entry), tcfstmp);
		}
		else
		{
			if(!tcfs_strgetpwnam(entry,&e))
				return 0;

			tcfs_key(user,old,e.upw,&key);

			while (strlen(new)<8)
				{ 
				 char tmp[UUKEYSIZE];
				 strcpy(tmp,new);
				 strcat(tmp,new);
				 strcat(new,tmp);
			        }
			
			while ((i*8)<=KEYSIZE)
			{
			 des_set_key ((des_cblock *)new, ks);
			 des_ecb_encrypt ((des_cblock *)(key+i*8), (des_cblock *)(key+i*8), ks, DES_ENCRYPT);
			 i++;
			}
			fprintf (tcfstmp, "%s:%s\n", user, 
				tcfs_encode(key, KEYSIZE));
		}			
	}
	
	unlink(TCFSPWDOLD);
	link(TCFSPWDFILE, TCFSPWDOLD);
	unlink(TCFSPWDFILE);
	link(TCFSPWDTMP, TCFSPWDFILE);
	unlink(TCFSPWDTMP); 
	chmod(TCFSPWDFILE, 0644);
	syslog(LOG_INFO,"tcfspasswd updated");	
	free(key);
	return 1;
}
